@@ -1,8 +1,8 @@
 from __future__ import annotations
 
+from importlib import metadata
 from typing import TYPE_CHECKING
 from typing import Any
-from typing import Callable
 
 import pytest
 
@@ -11,10 +11,10 @@ from poetry.core.packages.package import Package
 
 from poetry.plugins.application_plugin import ApplicationPlugin
 from poetry.plugins.plugin import Plugin
-from poetry.utils._compat import metadata
 
 
 if TYPE_CHECKING:
+    from collections.abc import Callable
     from os import PathLike
     from pathlib import Path
 
@@ -163,7 +163,7 @@ def test_show_displays_installed_plugins(
     tester.execute("")
 
     expected = """
-  • poetry-plugin (1.2.3)
+  - poetry-plugin (1.2.3)
       1 plugin and 1 application plugin
 """
 
@@ -189,7 +189,7 @@ def test_show_displays_installed_plugins_with_multiple_plugins(
     tester.execute("")
 
     expected = """
-  • poetry-plugin (1.2.3)
+  - poetry-plugin (1.2.3)
       2 plugins and 2 application plugins
 """
 
@@ -215,7 +215,7 @@ def test_show_displays_installed_plugins_with_dependencies(
     tester.execute("")
 
     expected = """
-  • poetry-plugin (1.2.3)
+  - poetry-plugin (1.2.3)
       1 plugin and 1 application plugin
 
       Dependencies
