@@ -7,25 +7,32 @@ from copy import deepcopy
 from hashlib import sha1
 from pathlib import Path
 from typing import TYPE_CHECKING
+from typing import TypedDict
+from urllib.parse import urlparse
+from urllib.parse import urlunparse
 
 import pytest
 
 from dulwich.client import HTTPUnauthorized
 from dulwich.client import get_transport_and_path
+from dulwich.config import CaseInsensitiveOrderedMultiDict
 from dulwich.config import ConfigFile
+from dulwich.refs import Ref
 from dulwich.repo import Repo
-from poetry.core.pyproject.toml import PyProjectTOML
 
 from poetry.console.exceptions import PoetryConsoleError
+from poetry.pyproject.toml import PyProjectTOML
 from poetry.utils.authenticator import Authenticator
 from poetry.vcs.git import Git
 from poetry.vcs.git.backend import GitRefSpec
 
 
 if TYPE_CHECKING:
-    from _pytest.tmpdir import TempdirFactory
+    from collections.abc import Iterator
+
     from dulwich.client import FetchPackResult
     from dulwich.client import GitClient
+    from pytest import TempPathFactory
     from pytest_mock import MockerFixture
 
     from tests.conftest import Config
@@ -36,6 +43,15 @@ if TYPE_CHECKING:
 pytestmark = pytest.mark.integration
 
 
+class GitCloneKwargs(TypedDict):
+    name: str | None
+    branch: str | None
+    tag: str | None
+    revision: str | None
+    source_root: Path | None
+    clean: bool
+
+
 @pytest.fixture(autouse=True)
 def git_mock() -> None:
     pass
@@ -65,7 +81,7 @@ REF_TO_REVISION_MAP = {
 
 @pytest.fixture
 def use_system_git_client(config: Config) -> None:
-    config.merge({"experimental": {"system-git-client": True}})
+    config.merge({"system-git-client": True})
 
 
 @pytest.fixture(scope="module")
@@ -79,9 +95,11 @@ def source_directory_name(source_url: str) -> str:
 
 
 @pytest.fixture(scope="module")
-def local_repo(tmpdir_factory: TempdirFactory, source_directory_name: str) -> Repo:
+def local_repo(
+    tmp_path_factory: TempPathFactory, source_directory_name: str
+) -> Iterator[Repo]:
     with Repo.init(
-        tmpdir_factory.mktemp("src") / source_directory_name, mkdir=True
+        str(tmp_path_factory.mktemp("src") / source_directory_name), mkdir=True
     ) as repo:
         yield repo
 
@@ -102,8 +120,9 @@ def remote_refs(_remote_refs: FetchPackResult) -> FetchPackResult:
 
 
 @pytest.fixture(scope="module")
-def remote_default_ref(_remote_refs: FetchPackResult) -> bytes:
-    return _remote_refs.symrefs[b"HEAD"]
+def remote_default_ref(_remote_refs: FetchPackResult) -> Ref:
+    ref: Ref = _remote_refs.symrefs[Ref(b"HEAD")]
+    return ref
 
 
 @pytest.fixture(scope="module")
@@ -112,38 +131,44 @@ def remote_default_branch(remote_default_ref: bytes) -> str:
 
 
 # Regression test for https://github.com/python-poetry/poetry/issues/6722
-def test_use_system_git_client_from_environment_variables():
-    os.environ["POETRY_EXPERIMENTAL_SYSTEM_GIT_CLIENT"] = "true"
+def test_use_system_git_client_from_environment_variables() -> None:
+    os.environ["POETRY_SYSTEM_GIT_CLIENT"] = "true"
 
     assert Git.is_using_legacy_client()
 
 
 def test_git_local_info(
-    source_url: str, remote_refs: FetchPackResult, remote_default_ref: bytes
+    source_url: str, remote_refs: FetchPackResult, remote_default_ref: Ref
 ) -> None:
     with Git.clone(url=source_url) as repo:
         info = Git.info(repo=repo)
         assert info.origin == source_url
-        assert info.revision == remote_refs.refs[remote_default_ref].decode("utf-8")
+        ref = remote_refs.refs[remote_default_ref]
+        assert ref is not None
+        assert info.revision == ref.decode("utf-8")
 
 
+@pytest.mark.parametrize(
+    "specification", [{}, {"revision": "HEAD"}, {"branch": "HEAD"}]
+)
 def test_git_clone_default_branch_head(
+    specification: GitCloneKwargs,
     source_url: str,
     remote_refs: FetchPackResult,
-    remote_default_ref: bytes,
+    remote_default_ref: Ref,
     mocker: MockerFixture,
-):
+) -> None:
     spy = mocker.spy(Git, "_clone")
     spy_legacy = mocker.spy(Git, "_clone_legacy")
 
-    with Git.clone(url=source_url) as repo:
+    with Git.clone(url=source_url, **specification) as repo:
         assert remote_refs.refs[remote_default_ref] == repo.head()
 
     spy_legacy.assert_not_called()
     spy.assert_called()
 
 
-def test_git_clone_fails_for_non_existent_branch(source_url: str):
+def test_git_clone_fails_for_non_existent_branch(source_url: str) -> None:
     branch = uuid.uuid4().hex
 
     with pytest.raises(PoetryConsoleError) as e:
@@ -152,7 +177,7 @@ def test_git_clone_fails_for_non_existent_branch(source_url: str):
     assert f"Failed to clone {source_url} at '{branch}'" in str(e.value)
 
 
-def test_git_clone_fails_for_non_existent_revision(source_url: str):
+def test_git_clone_fails_for_non_existent_revision(source_url: str) -> None:
     revision = sha1(uuid.uuid4().bytes).hexdigest()
 
     with pytest.raises(PoetryConsoleError) as e:
@@ -267,29 +292,37 @@ def test_git_clone_clones_submodules_with_relative_urls_and_explicit_base(
 def test_system_git_fallback_on_http_401(
     mocker: MockerFixture,
     source_url: str,
+    tmp_path: Path,
 ) -> None:
     spy = mocker.spy(Git, "_clone_legacy")
-    mocker.patch.object(Git, "_clone", side_effect=HTTPUnauthorized(None, None))
+    mocker.patch.object(
+        Git,
+        "_clone",
+        side_effect=HTTPUnauthorized(None, source_url),
+    )
 
-    with Git.clone(url=source_url, branch="0.1") as repo:
+    # use tmp_path for source_root to get a shorter path,
+    # because long paths can cause issues with the system git client on Windows
+    # despite of setting core.longpaths=true
+    with Git.clone(url=source_url, branch="0.1", source_root=tmp_path) as repo:
         path = Path(repo.path)
         assert_version(repo, BRANCH_TO_REVISION_MAP["0.1"])
 
     spy.assert_called_with(
         url="https://github.com/python-poetry/test-fixture-vcs-repository.git",
         target=path,
-        refspec=GitRefSpec(branch="0.1", revision=None, tag=None, ref=b"HEAD"),
+        refspec=GitRefSpec(branch="0.1", revision=None, tag=None, ref=Ref(b"HEAD")),
     )
     spy.assert_called_once()
 
 
 GIT_USERNAME = os.environ.get("POETRY_TEST_INTEGRATION_GIT_USERNAME")
 GIT_PASSWORD = os.environ.get("POETRY_TEST_INTEGRATION_GIT_PASSWORD")
-HTTP_AUTH_CREDENTIALS_AVAILABLE = not (GIT_USERNAME and GIT_PASSWORD)
+HTTP_AUTH_CREDENTIALS_UNAVAILABLE = not (GIT_USERNAME and GIT_PASSWORD)
 
 
 @pytest.mark.skipif(
-    HTTP_AUTH_CREDENTIALS_AVAILABLE,
+    HTTP_AUTH_CREDENTIALS_UNAVAILABLE,
     reason="HTTP authentication credentials not available",
 )
 def test_configured_repository_http_auth(
@@ -359,17 +392,22 @@ def test_username_password_parameter_is_not_passed_to_dulwich(
     spy_get_transport_and_path.assert_called_with(
         location=source_url,
         config=dummy_git_config,
+        username=None,
+        password=None,
     )
     spy_get_transport_and_path.assert_called_once()
 
 
 def test_system_git_called_when_configured(
-    mocker: MockerFixture, source_url: str, use_system_git_client: None
+    mocker: MockerFixture, source_url: str, use_system_git_client: None, tmp_path: Path
 ) -> None:
     spy_legacy = mocker.spy(Git, "_clone_legacy")
     spy = mocker.spy(Git, "_clone")
 
-    with Git.clone(url=source_url, branch="0.1") as repo:
+    # use tmp_path for source_root to get a shorter path,
+    # because long paths can cause issues with the system git client on Windows
+    # despite of setting core.longpaths=true
+    with Git.clone(url=source_url, branch="0.1", source_root=tmp_path) as repo:
         path = Path(repo.path)
         assert_version(repo, BRANCH_TO_REVISION_MAP["0.1"])
 
@@ -379,5 +417,35 @@ def test_system_git_called_when_configured(
     spy_legacy.assert_called_with(
         url=source_url,
         target=path,
-        refspec=GitRefSpec(branch="0.1", revision=None, tag=None, ref=b"HEAD"),
+        refspec=GitRefSpec(branch="0.1", revision=None, tag=None, ref=Ref(b"HEAD")),
     )
+
+
+def test_relative_submodules_with_ssh(
+    source_url: str, tmpdir: Path, mocker: MockerFixture
+) -> None:
+    target = tmpdir / "temp"
+    ssh_source_url = urlunparse(urlparse(source_url)._replace(scheme="ssh"))
+
+    repo_with_unresolved_submodules = Git._clone(
+        url=source_url,
+        refspec=GitRefSpec(branch="relative_submodule"),
+        target=target,
+    )
+
+    # construct fake git config
+    values = CaseInsensitiveOrderedMultiDict.make(
+        {b"url": ssh_source_url.encode("utf-8")}
+    )
+    fake_config = ConfigFile({(b"remote", b"origin"): values})
+    # trick Git into thinking remote.origin is an ssh url
+    mock_get_config = mocker.patch.object(repo_with_unresolved_submodules, "get_config")
+    mock_get_config.return_value = fake_config
+
+    submodules = Git._get_submodules(repo_with_unresolved_submodules)
+
+    assert [s.url for s in submodules] == [
+        "https://github.com/pypa/sample-namespace-packages.git",
+        "ssh://github.com/python-poetry/test-fixture-vcs-repository.git",
+        "ssh://github.com/python-poetry/test-fixture-vcs-repository.git",
+    ]
