@@ -1,5 +1,8 @@
 from __future__ import annotations
 
+import shutil
+
+from pathlib import Path
 from subprocess import CalledProcessError
 from typing import TYPE_CHECKING
 from typing import Any
@@ -7,6 +10,7 @@ from typing import Any
 import pytest
 
 from cleo.io.null_io import NullIO
+from packaging.utils import NormalizedName
 from packaging.utils import canonicalize_name
 from poetry.core.packages.dependency import Dependency
 from poetry.core.packages.directory_dependency import DirectoryDependency
@@ -21,7 +25,10 @@ from poetry.inspection.info import PackageInfo
 from poetry.packages import DependencyPackage
 from poetry.puzzle.provider import IncompatibleConstraintsError
 from poetry.puzzle.provider import Provider
+from poetry.repositories.cached_repository import CachedRepository
+from poetry.repositories.exceptions import PackageNotFoundError
 from poetry.repositories.repository import Repository
+from poetry.repositories.repository_pool import Priority
 from poetry.repositories.repository_pool import RepositoryPool
 from poetry.utils.env import EnvCommandError
 from poetry.utils.env import MockEnv as BaseMockEnv
@@ -29,6 +36,9 @@ from tests.helpers import get_dependency
 
 
 if TYPE_CHECKING:
+    from pathlib import Path
+
+    from poetry.core.constraints.version import Version
     from pytest_mock import MockerFixture
 
     from tests.types import FixtureDirGetter
@@ -42,6 +52,18 @@ class MockEnv(BaseMockEnv):
         raise EnvCommandError(CalledProcessError(1, "python", output=""))
 
 
+class MockCachedRepository(CachedRepository):
+    def _get_release_info(
+        self, name: NormalizedName, version: Version
+    ) -> dict[str, Any]:
+        raise NotImplementedError
+
+    def package(self, name: str, version: Version) -> Package:
+        package = super().package(name, version)
+        package._source_reference = self.name
+        return package
+
+
 @pytest.fixture
 def root() -> ProjectPackage:
     return ProjectPackage("root", "1.2.3")
@@ -65,6 +87,46 @@ def provider(root: ProjectPackage, pool: RepositoryPool) -> Provider:
     return Provider(root, pool, NullIO())
 
 
+@pytest.fixture
+def release_info() -> PackageInfo:
+    return PackageInfo(
+        name="mylib",
+        version="1.0",
+        summary="",
+        requires_dist=[],
+        requires_python=">=3.9",
+        files=[
+            {
+                "file": "mylib-1.0-py3-none-any.whl",
+                "hash": "sha256:dummyhashvalue1234567890abcdef",
+            },
+            {
+                "file": "mylib-1.0.tar.gz",
+                "hash": "sha256:anotherdummyhashvalueabcdef1234567890",
+            },
+        ],
+        cache_version=str(CachedRepository.CACHE_VERSION),
+    )
+
+
+@pytest.fixture
+def outdated_release_info() -> PackageInfo:
+    return PackageInfo(
+        name="mylib",
+        version="1.0",
+        summary="",
+        requires_dist=[],
+        requires_python=">=3.9",
+        files=[
+            {
+                "file": "mylib-1.0-py3-none-any.whl",
+                "hash": "sha256:dummyhashvalue1234567890abcdef",
+            }
+        ],
+        cache_version=str(CachedRepository.CACHE_VERSION),
+    )
+
+
 @pytest.mark.parametrize(
     "dependency, expected",
     [
@@ -177,10 +239,10 @@ def test_search_for_vcs_setup_egg_info(provider: Provider) -> None:
     assert package.name == "demo"
     assert package.version.text == "0.1.2"
 
-    required = [r for r in package.requires if not r.is_optional()]
-    optional = [r for r in package.requires if r.is_optional()]
-    assert required == [get_dependency("pendulum", ">=1.4.4")]
-    assert optional == [get_dependency("tomlkit"), get_dependency("cleo")]
+    required = {r for r in package.requires if not r.is_optional()}
+    optional = {r for r in package.requires if r.is_optional()}
+    assert required == {get_dependency("pendulum", ">=1.4.4")}
+    assert optional == {get_dependency("tomlkit"), get_dependency("cleo")}
     assert package.extras == {
         "foo": [get_dependency("cleo")],
         "bar": [get_dependency("tomlkit")],
@@ -197,10 +259,10 @@ def test_search_for_vcs_setup_egg_info_with_extras(provider: Provider) -> None:
     assert package.name == "demo"
     assert package.version.text == "0.1.2"
 
-    required = [r for r in package.requires if not r.is_optional()]
-    optional = [r for r in package.requires if r.is_optional()]
-    assert required == [get_dependency("pendulum", ">=1.4.4")]
-    assert optional == [get_dependency("tomlkit"), get_dependency("cleo")]
+    required = {r for r in package.requires if not r.is_optional()}
+    optional = {r for r in package.requires if r.is_optional()}
+    assert required == {get_dependency("pendulum", ">=1.4.4")}
+    assert optional == {get_dependency("tomlkit"), get_dependency("cleo")}
     assert package.extras == {
         "foo": [get_dependency("cleo")],
         "bar": [get_dependency("tomlkit")],
@@ -217,10 +279,10 @@ def test_search_for_vcs_read_setup(provider: Provider, mocker: MockerFixture) ->
     assert package.name == "demo"
     assert package.version.text == "0.1.2"
 
-    required = [r for r in package.requires if not r.is_optional()]
-    optional = [r for r in package.requires if r.is_optional()]
-    assert required == [get_dependency("pendulum", ">=1.4.4")]
-    assert optional == [get_dependency("tomlkit"), get_dependency("cleo")]
+    required = {r for r in package.requires if not r.is_optional()}
+    optional = {r for r in package.requires if r.is_optional()}
+    assert required == {get_dependency("pendulum", ">=1.4.4")}
+    assert optional == {get_dependency("tomlkit"), get_dependency("cleo")}
     assert package.extras == {
         "foo": [get_dependency("cleo")],
         "bar": [get_dependency("tomlkit")],
@@ -241,10 +303,10 @@ def test_search_for_vcs_read_setup_with_extras(
     assert package.name == "demo"
     assert package.version.text == "0.1.2"
 
-    required = [r for r in package.requires if not r.is_optional()]
-    optional = [r for r in package.requires if r.is_optional()]
-    assert required == [get_dependency("pendulum", ">=1.4.4")]
-    assert optional == [get_dependency("tomlkit"), get_dependency("cleo")]
+    required = {r for r in package.requires if not r.is_optional()}
+    optional = {r for r in package.requires if r.is_optional()}
+    assert required == {get_dependency("pendulum", ">=1.4.4")}
+    assert optional == {get_dependency("tomlkit"), get_dependency("cleo")}
 
 
 def test_search_for_vcs_read_setup_raises_error_if_no_version(
@@ -263,22 +325,22 @@ def test_search_for_vcs_read_setup_raises_error_if_no_version(
 
 @pytest.mark.parametrize("directory", ["demo", "non-canonical-name"])
 def test_search_for_directory_setup_egg_info(
-    provider: Provider, directory: str, fixture_dir: FixtureDirGetter
+    provider: Provider, directory: str, fixture_dir: FixtureDirGetter, tmp_path: Path
 ) -> None:
-    dependency = DirectoryDependency(
-        "demo",
-        fixture_dir("git") / "github.com" / "demo" / directory,
+    path = shutil.copytree(
+        fixture_dir("git") / "github.com" / "demo" / directory, tmp_path / "project"
     )
+    dependency = DirectoryDependency("demo", path)
 
     package = provider.search_for_direct_origin_dependency(dependency)
 
     assert package.name == "demo"
     assert package.version.text == "0.1.2"
 
-    required = [r for r in package.requires if not r.is_optional()]
-    optional = [r for r in package.requires if r.is_optional()]
-    assert required == [get_dependency("pendulum", ">=1.4.4")]
-    assert optional == [get_dependency("tomlkit"), get_dependency("cleo")]
+    required = {r for r in package.requires if not r.is_optional()}
+    optional = {r for r in package.requires if r.is_optional()}
+    assert required == {get_dependency("pendulum", ">=1.4.4")}
+    assert optional == {get_dependency("tomlkit"), get_dependency("cleo")}
     assert package.extras == {
         "foo": [get_dependency("cleo")],
         "bar": [get_dependency("tomlkit")],
@@ -286,23 +348,23 @@ def test_search_for_directory_setup_egg_info(
 
 
 def test_search_for_directory_setup_egg_info_with_extras(
-    provider: Provider, fixture_dir: FixtureDirGetter
+    provider: Provider, fixture_dir: FixtureDirGetter, tmp_path: Path
 ) -> None:
-    dependency = DirectoryDependency(
-        "demo",
-        fixture_dir("git") / "github.com" / "demo" / "demo",
-        extras=["foo"],
+    path = shutil.copytree(
+        fixture_dir("git") / "github.com" / "demo" / "demo", tmp_path / "project"
     )
 
+    dependency = DirectoryDependency("demo", path, extras=["foo"])
+
     package = provider.search_for_direct_origin_dependency(dependency)
 
     assert package.name == "demo"
     assert package.version.text == "0.1.2"
 
-    required = [r for r in package.requires if not r.is_optional()]
-    optional = [r for r in package.requires if r.is_optional()]
-    assert required == [get_dependency("pendulum", ">=1.4.4")]
-    assert optional == [get_dependency("tomlkit"), get_dependency("cleo")]
+    required = {r for r in package.requires if not r.is_optional()}
+    optional = {r for r in package.requires if r.is_optional()}
+    assert required == {get_dependency("pendulum", ">=1.4.4")}
+    assert optional == {get_dependency("tomlkit"), get_dependency("cleo")}
     assert package.extras == {
         "foo": [get_dependency("cleo")],
         "bar": [get_dependency("tomlkit")],
@@ -311,49 +373,52 @@ def test_search_for_directory_setup_egg_info_with_extras(
 
 @pytest.mark.parametrize("directory", ["demo", "non-canonical-name"])
 def test_search_for_directory_setup_with_base(
-    provider: Provider, directory: str, fixture_dir: FixtureDirGetter
+    provider: Provider, directory: str, fixture_dir: FixtureDirGetter, tmp_path: Path
 ) -> None:
-    dependency = DirectoryDependency(
-        "demo",
-        fixture_dir("git") / "github.com" / "demo" / directory,
-        base=fixture_dir("git") / "github.com" / "demo" / directory,
+    path = shutil.copytree(
+        fixture_dir("git") / "github.com" / "demo" / directory, tmp_path / "project"
     )
 
+    dependency = DirectoryDependency("demo", path, base=path)
+
     package = provider.search_for_direct_origin_dependency(dependency)
 
     assert package.name == "demo"
     assert package.version.text == "0.1.2"
 
-    required = [r for r in package.requires if not r.is_optional()]
-    optional = [r for r in package.requires if r.is_optional()]
-    assert required == [get_dependency("pendulum", ">=1.4.4")]
-    assert optional == [get_dependency("tomlkit"), get_dependency("cleo")]
+    required = {r for r in package.requires if not r.is_optional()}
+    optional = {r for r in package.requires if r.is_optional()}
+    assert required == {get_dependency("pendulum", ">=1.4.4")}
+    assert optional == {get_dependency("tomlkit"), get_dependency("cleo")}
     assert package.extras == {
         "foo": [get_dependency("cleo")],
         "bar": [get_dependency("tomlkit")],
     }
-    assert package.root_dir == (fixture_dir("git") / "github.com" / "demo" / directory)
+    assert package.root_dir == path
 
 
 def test_search_for_directory_setup_read_setup(
-    provider: Provider, mocker: MockerFixture, fixture_dir: FixtureDirGetter
+    provider: Provider,
+    mocker: MockerFixture,
+    fixture_dir: FixtureDirGetter,
+    tmp_path: Path,
 ) -> None:
     mocker.patch("poetry.utils.env.EnvManager.get", return_value=MockEnv())
 
-    dependency = DirectoryDependency(
-        "demo",
-        fixture_dir("git") / "github.com" / "demo" / "demo",
+    path = shutil.copytree(
+        fixture_dir("git") / "github.com" / "demo" / "demo", tmp_path / "project"
     )
+    dependency = DirectoryDependency("demo", path)
 
     package = provider.search_for_direct_origin_dependency(dependency)
 
     assert package.name == "demo"
     assert package.version.text == "0.1.2"
 
-    required = [r for r in package.requires if not r.is_optional()]
-    optional = [r for r in package.requires if r.is_optional()]
-    assert required == [get_dependency("pendulum", ">=1.4.4")]
-    assert optional == [get_dependency("tomlkit"), get_dependency("cleo")]
+    required = {r for r in package.requires if not r.is_optional()}
+    optional = {r for r in package.requires if r.is_optional()}
+    assert required == {get_dependency("pendulum", ">=1.4.4")}
+    assert optional == {get_dependency("tomlkit"), get_dependency("cleo")}
     assert package.extras == {
         "foo": [get_dependency("cleo")],
         "bar": [get_dependency("tomlkit")],
@@ -361,25 +426,28 @@ def test_search_for_directory_setup_read_setup(
 
 
 def test_search_for_directory_setup_read_setup_with_extras(
-    provider: Provider, mocker: MockerFixture, fixture_dir: FixtureDirGetter
+    provider: Provider,
+    mocker: MockerFixture,
+    fixture_dir: FixtureDirGetter,
+    tmp_path: Path,
 ) -> None:
     mocker.patch("poetry.utils.env.EnvManager.get", return_value=MockEnv())
 
-    dependency = DirectoryDependency(
-        "demo",
-        fixture_dir("git") / "github.com" / "demo" / "demo",
-        extras=["foo"],
+    path = shutil.copytree(
+        fixture_dir("git") / "github.com" / "demo" / "demo", tmp_path / "project"
     )
 
+    dependency = DirectoryDependency("demo", path, extras=["foo"])
+
     package = provider.search_for_direct_origin_dependency(dependency)
 
     assert package.name == "demo"
     assert package.version.text == "0.1.2"
 
-    required = [r for r in package.requires if not r.is_optional()]
-    optional = [r for r in package.requires if r.is_optional()]
-    assert required == [get_dependency("pendulum", ">=1.4.4")]
-    assert optional == [get_dependency("tomlkit"), get_dependency("cleo")]
+    required = {r for r in package.requires if not r.is_optional()}
+    optional = {r for r in package.requires if r.is_optional()}
+    assert required == {get_dependency("pendulum", ">=1.4.4")}
+    assert optional == {get_dependency("tomlkit"), get_dependency("cleo")}
     assert package.extras == {
         "foo": [get_dependency("cleo")],
         "bar": [get_dependency("tomlkit")],
@@ -387,13 +455,15 @@ def test_search_for_directory_setup_read_setup_with_extras(
 
 
 def test_search_for_directory_setup_read_setup_with_no_dependencies(
-    provider: Provider, fixture_dir: FixtureDirGetter
+    provider: Provider, fixture_dir: FixtureDirGetter, tmp_path: Path
 ) -> None:
-    dependency = DirectoryDependency(
-        "demo",
+    path = shutil.copytree(
         fixture_dir("git") / "github.com" / "demo" / "no-dependencies",
+        tmp_path / "project",
     )
 
+    dependency = DirectoryDependency("demo", path)
+
     package = provider.search_for_direct_origin_dependency(dependency)
 
     assert package.name == "demo"
@@ -403,29 +473,28 @@ def test_search_for_directory_setup_read_setup_with_no_dependencies(
 
 
 def test_search_for_directory_poetry(
-    provider: Provider, fixture_dir: FixtureDirGetter
+    provider: Provider, fixture_dir: FixtureDirGetter, tmp_path: Path
 ) -> None:
-    dependency = DirectoryDependency(
-        "project-with-extras",
-        fixture_dir("project_with_extras"),
-    )
+    path = shutil.copytree(fixture_dir("project_with_extras"), tmp_path / "project")
+
+    dependency = DirectoryDependency("project-with-extras", path)
 
     package = provider.search_for_direct_origin_dependency(dependency)
 
     assert package.name == "project-with-extras"
     assert package.version.text == "1.2.3"
 
-    required = [
+    required = {
         r for r in sorted(package.requires, key=lambda r: r.name) if not r.is_optional()
-    ]
-    optional = [
+    }
+    optional = {
         r for r in sorted(package.requires, key=lambda r: r.name) if r.is_optional()
-    ]
-    assert required == []
-    assert optional == [
+    }
+    assert not required
+    assert optional == {
         get_dependency("cachy", ">=0.2.0"),
         get_dependency("pendulum", ">=1.4.4"),
-    ]
+    }
     extras_a = canonicalize_name("extras-a")
     extras_b = canonicalize_name("extras-b")
     assert set(package.extras) == {extras_a, extras_b}
@@ -434,30 +503,28 @@ def test_search_for_directory_poetry(
 
 
 def test_search_for_directory_poetry_with_extras(
-    provider: Provider, fixture_dir: FixtureDirGetter
+    provider: Provider, fixture_dir: FixtureDirGetter, tmp_path: Path
 ) -> None:
-    dependency = DirectoryDependency(
-        "project-with-extras",
-        fixture_dir("project_with_extras"),
-        extras=["extras_a"],
-    )
+    path = shutil.copytree(fixture_dir("project_with_extras"), tmp_path / "project")
+
+    dependency = DirectoryDependency("project-with-extras", path, extras=["extras_a"])
 
     package = provider.search_for_direct_origin_dependency(dependency)
 
     assert package.name == "project-with-extras"
     assert package.version.text == "1.2.3"
 
-    required = [
+    required = {
         r for r in sorted(package.requires, key=lambda r: r.name) if not r.is_optional()
-    ]
-    optional = [
+    }
+    optional = {
         r for r in sorted(package.requires, key=lambda r: r.name) if r.is_optional()
-    ]
-    assert required == []
-    assert optional == [
+    }
+    assert not required
+    assert optional == {
         get_dependency("cachy", ">=0.2.0"),
         get_dependency("pendulum", ">=1.4.4"),
-    ]
+    }
     extras_a = canonicalize_name("extras-a")
     extras_b = canonicalize_name("extras-b")
     assert set(package.extras) == {extras_a, extras_b}
@@ -477,18 +544,25 @@ def test_search_for_file_sdist(
 
     assert package.name == "demo"
     assert package.version.text == "0.1.0"
+    assert package.files == [
+        {
+            "file": "demo-0.1.0.tar.gz",
+            "hash": "sha256:9fa123ad707a5c6c944743bf3e11a0e80d86cb518d3cf25320866ca3ef43e2ad",
+            "size": 1003,
+        }
+    ]
 
-    required = [
+    required = {
         r for r in sorted(package.requires, key=lambda r: r.name) if not r.is_optional()
-    ]
-    optional = [
+    }
+    optional = {
         r for r in sorted(package.requires, key=lambda r: r.name) if r.is_optional()
-    ]
-    assert required == [get_dependency("pendulum", ">=1.4.4")]
-    assert optional == [
+    }
+    assert required == {get_dependency("pendulum", ">=1.4.4")}
+    assert optional == {
         get_dependency("cleo"),
         get_dependency("tomlkit"),
-    ]
+    }
     assert package.extras == {
         "foo": [get_dependency("cleo")],
         "bar": [get_dependency("tomlkit")],
@@ -508,18 +582,25 @@ def test_search_for_file_sdist_with_extras(
 
     assert package.name == "demo"
     assert package.version.text == "0.1.0"
+    assert package.files == [
+        {
+            "file": "demo-0.1.0.tar.gz",
+            "hash": "sha256:9fa123ad707a5c6c944743bf3e11a0e80d86cb518d3cf25320866ca3ef43e2ad",
+            "size": 1003,
+        }
+    ]
 
-    required = [
+    required = {
         r for r in sorted(package.requires, key=lambda r: r.name) if not r.is_optional()
-    ]
-    optional = [
+    }
+    optional = {
         r for r in sorted(package.requires, key=lambda r: r.name) if r.is_optional()
-    ]
-    assert required == [get_dependency("pendulum", ">=1.4.4")]
-    assert optional == [
+    }
+    assert required == {get_dependency("pendulum", ">=1.4.4")}
+    assert optional == {
         get_dependency("cleo"),
         get_dependency("tomlkit"),
-    ]
+    }
     assert package.extras == {
         "foo": [get_dependency("cleo")],
         "bar": [get_dependency("tomlkit")],
@@ -538,18 +619,25 @@ def test_search_for_file_wheel(
 
     assert package.name == "demo"
     assert package.version.text == "0.1.0"
+    assert package.files == [
+        {
+            "file": "demo-0.1.0-py2.py3-none-any.whl",
+            "hash": "sha256:70e704135718fffbcbf61ed1fc45933cfd86951a744b681000eaaa75da31f17a",
+            "size": 1116,
+        }
+    ]
 
-    required = [
+    required = {
         r for r in sorted(package.requires, key=lambda r: r.name) if not r.is_optional()
-    ]
-    optional = [
+    }
+    optional = {
         r for r in sorted(package.requires, key=lambda r: r.name) if r.is_optional()
-    ]
-    assert required == [get_dependency("pendulum", ">=1.4.4")]
-    assert optional == [
+    }
+    assert required == {get_dependency("pendulum", ">=1.4.4")}
+    assert optional == {
         get_dependency("cleo"),
         get_dependency("tomlkit"),
-    ]
+    }
     assert package.extras == {
         "foo": [get_dependency("cleo")],
         "bar": [get_dependency("tomlkit")],
@@ -569,18 +657,25 @@ def test_search_for_file_wheel_with_extras(
 
     assert package.name == "demo"
     assert package.version.text == "0.1.0"
+    assert package.files == [
+        {
+            "file": "demo-0.1.0-py2.py3-none-any.whl",
+            "hash": "sha256:70e704135718fffbcbf61ed1fc45933cfd86951a744b681000eaaa75da31f17a",
+            "size": 1116,
+        }
+    ]
 
-    required = [
+    required = {
         r for r in sorted(package.requires, key=lambda r: r.name) if not r.is_optional()
-    ]
-    optional = [
+    }
+    optional = {
         r for r in sorted(package.requires, key=lambda r: r.name) if r.is_optional()
-    ]
-    assert required == [get_dependency("pendulum", ">=1.4.4")]
-    assert optional == [
+    }
+    assert required == {get_dependency("pendulum", ">=1.4.4")}
+    assert optional == {
         get_dependency("cleo"),
         get_dependency("tomlkit"),
-    ]
+    }
     assert package.extras == {
         "foo": [get_dependency("cleo")],
         "bar": [get_dependency("tomlkit")],
@@ -770,7 +865,7 @@ def test_complete_package_fetches_optional_vcs_dependency_only_if_requested(
     )
     package = Package("A", "1.0", features=["foo"] if with_extra else [])
     package.add_dependency(optional_vcs_dependency)
-    package.extras[canonicalize_name("foo")] = [optional_vcs_dependency]
+    package.extras = {canonicalize_name("foo"): [optional_vcs_dependency]}
     repository.add_package(package)
 
     spy = mocker.spy(provider, "_search_for_vcs")
@@ -783,6 +878,104 @@ def test_complete_package_fetches_optional_vcs_dependency_only_if_requested(
         spy.assert_not_called()
 
 
+def test_complete_package_finds_locked_package_in_explicit_source(
+    root: ProjectPackage, pool: RepositoryPool
+) -> None:
+    package = Package("a", "1.0", source_reference="explicit")
+    explicit_repo = Repository("explicit")
+    explicit_repo.add_package(package)
+    pool.add_repository(explicit_repo, priority=Priority.EXPLICIT)
+
+    root_dependency = get_dependency("a", ">0")
+    root_dependency.source_name = "explicit"
+    root.add_dependency(root_dependency)
+    locked_package = Package("a", "1.0", source_reference="explicit")
+    provider = Provider(root, pool, NullIO(), locked=[locked_package])
+    provider.complete_package(DependencyPackage(root.to_dependency(), root))
+
+    # transitive dependency without explicit source
+    dependency = get_dependency("a", ">=1")
+
+    locked = provider.get_locked(dependency)
+    assert locked is not None
+    provider.complete_package(locked)  # must not fail
+
+
+def test_complete_package_finds_locked_package_in_other_source(
+    root: ProjectPackage, repository: Repository, pool: RepositoryPool
+) -> None:
+    package = Package("a", "1.0")
+    repository.add_package(package)
+    explicit_repo = Repository("explicit")
+    pool.add_repository(explicit_repo)
+
+    root_dependency = get_dependency("a", ">0")  # no explicit source
+    root.add_dependency(root_dependency)
+    locked_package = Package("a", "1.0", source_reference="explicit")  # explicit source
+    provider = Provider(root, pool, NullIO(), locked=[locked_package])
+    provider.complete_package(DependencyPackage(root.to_dependency(), root))
+
+    # transitive dependency without explicit source
+    dependency = get_dependency("a", ">=1")
+
+    locked = provider.get_locked(dependency)
+    assert locked is not None
+    provider.complete_package(locked)  # must not fail
+
+
+def test_complete_package_raises_packagenotfound_if_locked_source_not_available(
+    root: ProjectPackage, pool: RepositoryPool, provider: Provider
+) -> None:
+    locked_package = Package("a", "1.0", source_reference="outdated")
+    provider = Provider(root, pool, NullIO(), locked=[locked_package])
+    provider.complete_package(DependencyPackage(root.to_dependency(), root))
+
+    # transitive dependency without explicit source
+    dependency = get_dependency("a", ">=1")
+
+    locked = provider.get_locked(dependency)
+    assert locked is not None
+    with pytest.raises(PackageNotFoundError):
+        provider.complete_package(locked)
+
+
+def test_complete_package_outdated_cache(
+    root: ProjectPackage,
+    release_info: PackageInfo,
+    outdated_release_info: PackageInfo,
+    mocker: MockerFixture,
+) -> None:
+    repo = MockCachedRepository("repo")
+    pool = RepositoryPool()
+    pool.add_repository(repo)
+    provider = Provider(root, pool, NullIO())
+    pool_refresh_spy = mocker.spy(provider.pool, "refresh")
+
+    assert release_info.name is not None
+    assert release_info.version is not None
+    package = Package(release_info.name, release_info.version)
+    package.files = release_info.files  # up-to-date files from lock
+
+    # trigger caching of outdated info
+    repo._get_release_info = lambda name, version: outdated_release_info.asdict()  # type: ignore[method-assign]
+    assert len(repo.package(package.name, package.version).files) == 1
+
+    # additional files uploaded -> refresh needed
+    repo._get_release_info = lambda name, version: release_info.asdict()  # type: ignore[method-assign]
+    complete_package = provider.complete_package(
+        DependencyPackage(package.to_dependency(), package)
+    )
+    assert len(complete_package.package.files) == 2
+    assert pool_refresh_spy.call_count == 1
+
+    # cache should have been updated -> no additional refresh needed
+    complete_package = provider.complete_package(
+        DependencyPackage(package.to_dependency(), package)
+    )
+    assert len(complete_package.package.files) == 2
+    assert pool_refresh_spy.call_count == 1
+
+
 def test_source_dependency_is_satisfied_by_direct_origin(
     provider: Provider, repository: Repository
 ) -> None:
