@@ -2,8 +2,12 @@ from __future__ import annotations
 
 from typing import TYPE_CHECKING
 
+import pytest
+
 from packaging.utils import canonicalize_name
+from poetry.core.constraints.version import parse_constraint
 from poetry.core.packages.dependency import Dependency
+from poetry.core.packages.dependency_group import MAIN_GROUP
 from poetry.core.packages.package import Package
 from poetry.core.version.markers import AnyMarker
 from poetry.core.version.markers import parse_marker
@@ -23,6 +27,9 @@ if TYPE_CHECKING:
     from poetry.core.packages.project_package import ProjectPackage
 
 
+DEV_GROUP = canonicalize_name("dev")
+
+
 def dep(
     name: str,
     marker: str = "",
@@ -33,7 +40,7 @@ def dep(
     d = Dependency(name, "1", groups=groups, extras=extras)
     d._in_extras = [canonicalize_name(e) for e in in_extras]
     if marker:
-        d.marker = marker  # type: ignore[assignment]
+        d.marker = marker
     return d
 
 
@@ -213,16 +220,13 @@ def test_propagate_markers_with_root_extra(
     assert tm(result[package]) == {}
     assert tm(result[a]) == {"main": 'extra == "foo"'}
     assert tm(result[b]) == {
-        "main": (
-            'extra == "foo" and sys_platform == "linux"'
-            ' or extra == "bar" and sys_platform == "linux"'
-        )
+        "main": 'sys_platform == "linux" and (extra == "foo" or extra == "bar")',
     }
     assert tm(result[c]) == {"main": 'extra == "foo" and python_version == "3.8"'}
     assert tm(result[d]) == {
         "main": (
-            'extra == "foo" and sys_platform == "linux" and python_version == "3.9"'
-            ' or extra == "bar" and sys_platform == "linux" and python_version == "3.9"'
+            'sys_platform == "linux" and (extra == "foo" or extra == "bar")'
+            ' and python_version == "3.9"'
         )
     }
 
@@ -332,6 +336,50 @@ def test_propagate_markers_for_groups2(package: ProjectPackage, solver: Solver)
     }
 
 
+def test_propagate_markers_for_groups_same_dep(
+    package: ProjectPackage, solver: Solver
+) -> None:
+    a = Package("a", "1")
+    b = Package("b", "1")
+    package.add_dependency(dep("a", 'sys_platform == "win32"', groups=["main"]))
+    package.add_dependency(dep("a", 'sys_platform == "linux"', groups=["dev"]))
+    a.add_dependency(dep("b", 'python_version == "3.8"'))
+
+    packages = [package, a, b]
+    result = solver._aggregate_solved_packages(packages)
+
+    assert len(result) == len(packages)
+    assert result[package].groups == set()
+    assert result[a].groups == {"main", "dev"}
+    assert result[b].groups == {"main", "dev"}
+    assert tm(result[package]) == {}
+    assert tm(result[a]) == {
+        "main": 'sys_platform == "win32"',
+        "dev": 'sys_platform == "linux"',
+    }
+    assert tm(result[b]) == {
+        "main": 'sys_platform == "win32" and python_version == "3.8"',
+        "dev": 'sys_platform == "linux" and python_version == "3.8"',
+    }
+
+
+def test_propagate_markers_for_groups_with_extra(
+    package: ProjectPackage, solver: Solver
+) -> None:
+    a = Package("a", "1")
+    package.add_dependency(dep("a", groups=["main"], in_extras=["foo"]))
+    package.add_dependency(dep("a", groups=["dev"]))
+
+    packages = [package, a]
+    result = solver._aggregate_solved_packages(packages)
+
+    assert len(result) == len(packages)
+    assert result[package].groups == set()
+    assert result[a].groups == {"main", "dev"}
+    assert tm(result[package]) == {}
+    assert tm(result[a]) == {"main": 'extra == "foo"', "dev": ""}
+
+
 def test_propagate_markers_with_cycle(package: ProjectPackage, solver: Solver) -> None:
     a = Package("a", "1")
     b = Package("b", "1")
@@ -369,7 +417,9 @@ def test_merge_override_packages_restricted(package: ProjectPackage) -> None:
                 {package: {"a": dep("b", 'python_version < "3.9"')}},
                 {
                     a: TransitivePackageInfo(
-                        0, {"main"}, {"main": parse_marker("sys_platform == 'win32'")}
+                        0,
+                        {MAIN_GROUP},
+                        {MAIN_GROUP: parse_marker("sys_platform == 'win32'")},
                     )
                 },
             ),
@@ -377,11 +427,14 @@ def test_merge_override_packages_restricted(package: ProjectPackage) -> None:
                 {package: {"a": dep("b", 'python_version >= "3.9"')}},
                 {
                     a: TransitivePackageInfo(
-                        0, {"main"}, {"main": parse_marker("sys_platform == 'linux'")}
+                        0,
+                        {MAIN_GROUP},
+                        {MAIN_GROUP: parse_marker("sys_platform == 'linux'")},
                     )
                 },
             ),
-        ]
+        ],
+        parse_constraint("*"),
     )
     assert len(packages) == 1
     assert packages[a].groups == {"main"}
@@ -403,7 +456,9 @@ def test_merge_override_packages_extras(package: ProjectPackage) -> None:
                 {package: {"a": dep("b", 'python_version < "3.9" and extra == "foo"')}},
                 {
                     a: TransitivePackageInfo(
-                        0, {"main"}, {"main": parse_marker("sys_platform == 'win32'")}
+                        0,
+                        {MAIN_GROUP},
+                        {MAIN_GROUP: parse_marker("sys_platform == 'win32'")},
                     )
                 },
             ),
@@ -415,11 +470,14 @@ def test_merge_override_packages_extras(package: ProjectPackage) -> None:
                 },
                 {
                     a: TransitivePackageInfo(
-                        0, {"main"}, {"main": parse_marker("sys_platform == 'linux'")}
+                        0,
+                        {MAIN_GROUP},
+                        {MAIN_GROUP: parse_marker("sys_platform == 'linux'")},
                     )
                 },
             ),
-        ]
+        ],
+        parse_constraint("*"),
     )
     assert len(packages) == 1
     assert packages[a].groups == {"main"}
@@ -431,6 +489,43 @@ def test_merge_override_packages_extras(package: ProjectPackage) -> None:
     }
 
 
+@pytest.mark.parametrize(
+    ("python_constraint", "expected"),
+    [
+        (">=3.8", 'python_version > "3.8" or sys_platform != "linux"'),
+        (">=3.9", ""),
+    ],
+)
+def test_merge_override_packages_python_constraint(
+    package: ProjectPackage, python_constraint: str, expected: str
+) -> None:
+    """The resulting marker depends on the project's python constraint."""
+    a = Package("a", "1")
+
+    packages = merge_override_packages(
+        [
+            (
+                {
+                    package: {
+                        "a": dep(
+                            "b", "sys_platform == 'linux' and python_version > '3.8'"
+                        )
+                    }
+                },
+                {a: TransitivePackageInfo(0, {MAIN_GROUP}, {MAIN_GROUP: AnyMarker()})},
+            ),
+            (
+                {package: {"a": dep("b", "sys_platform != 'linux'")}},
+                {a: TransitivePackageInfo(0, {MAIN_GROUP}, {MAIN_GROUP: AnyMarker()})},
+            ),
+        ],
+        parse_constraint(python_constraint),
+    )
+    assert len(packages) == 1
+    assert packages[a].groups == {"main"}
+    assert tm(packages[a]) == {"main": expected}
+
+
 def test_merge_override_packages_multiple_deps(package: ProjectPackage) -> None:
     """All override markers should be intersected."""
     a = Package("a", "1")
@@ -445,18 +540,16 @@ def test_merge_override_packages_multiple_deps(package: ProjectPackage) -> None:
                     },
                     a: {"e": dep("f", 'python_version >= "3.8"')},
                 },
-                {a: TransitivePackageInfo(0, {"main"}, {"main": AnyMarker()})},
+                {a: TransitivePackageInfo(0, {MAIN_GROUP}, {MAIN_GROUP: AnyMarker()})},
             ),
-        ]
+        ],
+        parse_constraint("*"),
     )
 
     assert len(packages) == 1
     assert packages[a].groups == {"main"}
     assert tm(packages[a]) == {
-        "main": (
-            'python_version < "3.9" and sys_platform == "linux"'
-            ' and python_version >= "3.8"'
-        )
+        "main": 'python_version == "3.8" and sys_platform == "linux"'
     }
 
 
@@ -470,14 +563,16 @@ def test_merge_override_packages_groups(package: ProjectPackage) -> None:
                 {package: {"a": dep("b", 'python_version < "3.9"')}},
                 {
                     a: TransitivePackageInfo(
-                        0, {"main"}, {"main": parse_marker("sys_platform == 'win32'")}
+                        0,
+                        {MAIN_GROUP},
+                        {MAIN_GROUP: parse_marker("sys_platform == 'win32'")},
                     ),
                     b: TransitivePackageInfo(
                         0,
-                        {"main", "dev"},
+                        {MAIN_GROUP, DEV_GROUP},
                         {
-                            "main": parse_marker("sys_platform == 'win32'"),
-                            "dev": parse_marker("sys_platform == 'linux'"),
+                            MAIN_GROUP: parse_marker("sys_platform == 'win32'"),
+                            DEV_GROUP: parse_marker("sys_platform == 'linux'"),
                         },
                     ),
                 },
@@ -486,19 +581,22 @@ def test_merge_override_packages_groups(package: ProjectPackage) -> None:
                 {package: {"a": dep("b", 'python_version >= "3.9"')}},
                 {
                     a: TransitivePackageInfo(
-                        0, {"dev"}, {"dev": parse_marker("sys_platform == 'linux'")}
+                        0,
+                        {DEV_GROUP},
+                        {DEV_GROUP: parse_marker("sys_platform == 'linux'")},
                     ),
                     b: TransitivePackageInfo(
                         0,
-                        {"main", "dev"},
+                        {MAIN_GROUP, DEV_GROUP},
                         {
-                            "main": parse_marker("platform_machine == 'amd64'"),
-                            "dev": parse_marker("platform_machine == 'aarch64'"),
+                            MAIN_GROUP: parse_marker("platform_machine == 'amd64'"),
+                            DEV_GROUP: parse_marker("platform_machine == 'aarch64'"),
                         },
                     ),
                 },
             ),
-        ]
+        ],
+        parse_constraint("*"),
     )
     assert len(packages) == 2
     assert packages[a].groups == {"main", "dev"}
@@ -519,4 +617,53 @@ def test_merge_override_packages_groups(package: ProjectPackage) -> None:
     }
 
 
+def test_merge_override_packages_shortcut(package: ProjectPackage) -> None:
+    a = Package("a", "1")
+    common_marker = (
+        'extra == "test" and sys_platform == "win32" or platform_system == "Windows"'
+        ' or sys_platform == "linux" and extra == "stretch"'
+    )
+    override_marker1 = 'python_version >= "3.12" and platform_system != "Emscripten"'
+    override_marker2 = 'python_version >= "3.12" and platform_system == "Emscripten"'
+
+    packages = merge_override_packages(
+        [
+            (
+                {package: {"a": dep("b", override_marker1)}},
+                {
+                    a: TransitivePackageInfo(
+                        0,
+                        {MAIN_GROUP},
+                        {
+                            MAIN_GROUP: parse_marker(
+                                f"{override_marker1} and ({common_marker})"
+                            )
+                        },
+                    )
+                },
+            ),
+            (
+                {package: {"a": dep("b", override_marker2)}},
+                {
+                    a: TransitivePackageInfo(
+                        0,
+                        {MAIN_GROUP},
+                        {
+                            MAIN_GROUP: parse_marker(
+                                f"{override_marker2} and ({common_marker})"
+                            )
+                        },
+                    )
+                },
+            ),
+        ],
+        parse_constraint("*"),
+    )
+    assert len(packages) == 1
+    assert packages[a].groups == {"main"}
+    assert tm(packages[a]) == {
+        "main": f'({common_marker}) and python_version >= "3.12"'
+    }
+
+
 # TODO: root extras
